# flake8: noqa: F401
# ruff: noqa: F401
from galileo_core.helpers.dependencies import is_dependency_available
from galileo_core.helpers.project import get_project, get_project_from_name

from galileo_protect.invoke import ainvoke, invoke
from galileo_protect.project import create_project
from galileo_protect.schemas import (
    OverrideAction,
    PassthroughAction,
    Payload,
    Request,
    Response,
    Rule,
    RuleMetrics,
    RuleOperator,
    Ruleset,
    Stage,
)
from galileo_protect.stage import create_stage, pause_stage, resume_stage

if is_dependency_available("langchain_core"):
    from galileo_protect.langchain import ProtectParser, ProtectTool


__version__ = "0.7.0"



from typing import Optional, Sequence, Type

from galileo_core.schemas.protect.response import Response
from langchain_core.runnables.base import Runnable
from langchain_core.tools import BaseTool
from pydantic import UUID4, BaseModel, ConfigDict
from pydantic.v1 import BaseModel as BaseModelV1

from galileo_protect.constants.invoke import TIMEOUT
from galileo_protect.helpers.config import ProtectConfig
from galileo_protect.invoke import ainvoke, invoke
from galileo_protect.schemas import Payload, Ruleset


class PayloadV1(BaseModelV1):
    input: Optional[str] = None
    output: Optional[str] = None


class ProtectTool(BaseTool):
    name: str = "GalileoProtect"
    description: str = (
        "Protect your LLM applications from harmful content using Galileo Protect. "
        "This tool is a wrapper around Galileo's Protect API, can be used to scan text "
        "for harmful content, and can be used to trigger actions based on the results."
        "The tool can be used synchronously or asynchronously, on the input text or output text,"
        "and can be configured with a set of rulesets to evaluate on."
    )
    args_schema: Type[BaseModelV1] = PayloadV1

    prioritized_rulesets: Optional[Sequence[Ruleset]] = None
    project_id: Optional[UUID4] = None
    stage_name: Optional[str] = None
    stage_id: Optional[UUID4] = None
    timeout: float = TIMEOUT
    config: Optional[ProtectConfig] = None

    def _run(self, input: Optional[str] = None, output: Optional[str] = None) -> str:
        """
        Apply the tool synchronously.

        We serialize the response to JSON because that's what `langchain_core` expects
        for tools.
        """
        payload = Payload(input=input, output=output)
        return invoke(
            payload=payload,
            prioritized_rulesets=self.prioritized_rulesets,
            project_id=self.project_id,
            stage_name=self.stage_name,
            stage_id=self.stage_id,
            timeout=self.timeout,
            config=self.config,
        ).model_dump_json()

    async def _arun(self, input: Optional[str] = None, output: Optional[str] = None) -> str:
        """
        Apply the tool asynchronously.

        We serialize the response to JSON because that's what `langchain_core` expects
        for tools.
        """
        payload = Payload(input=input, output=output)
        response = await ainvoke(
            prioritized_rulesets=self.prioritized_rulesets,
            payload=payload,
            project_id=self.project_id,
            stage_name=self.stage_name,
            stage_id=self.stage_id,
            timeout=self.timeout,
            config=self.config,
        )
        return response.model_dump_json()


class ProtectParser(BaseModel):
    chain: Runnable

    model_config = ConfigDict(arbitrary_types_allowed=True)

    def parser(self, output: str) -> str:
        try:
            response = Response.model_validate_json(output)
        except Exception:
            return self.chain.invoke(output)
        text = response.text
        if response.status == "TRIGGERED":
            return text
        else:
            return self.chain.invoke(text)



from enum import Enum


class RuleMetrics(str, Enum):
    input_pii = "input_pii"
    input_sexist = "input_sexist"
    input_tone = "input_tone"
    input_toxicity = "input_toxicity"
    prompt_injection = "prompt_injection"
    pii = "pii"
    sexist = "sexist"
    tone = "tone"
    toxicity = "toxicity"



from typing import Optional

from galileo_core.utils.name import ts_name
from pydantic import UUID4
from requests import post, put

from galileo_protect.constants.routes import Routes
from galileo_protect.helpers.config import ProtectConfig
from galileo_protect.schemas import Action, PassthroughAction, Stage
from galileo_protect.schemas.stage import StageResponse


def create_stage(
    project_id: Optional[UUID4] = None,
    name: Optional[str] = None,
    description: Optional[str] = None,
    action: Action = PassthroughAction(),
    action_enabled: bool = False,
    config: Optional[ProtectConfig] = None,
) -> StageResponse:
    config = config or ProtectConfig.get()
    project_id = project_id or config.project_id
    if project_id is None:
        raise ValueError("Project ID must be provided to create a stage.")
    name = name or ts_name("stage")
    stage = StageResponse.model_validate(
        config.api_client.request(
            post,
            Routes.stages.format(project_id=project_id),
            json=Stage(
                name=name, project_id=project_id, description=description, action=action, action_enabled=action_enabled
            ).model_dump(mode="json"),
        )
    )
    config.project_id = project_id
    config.stage_id = stage.id
    config.stage_name = stage.name
    config.write()
    return stage


def pause_stage(
    project_id: Optional[UUID4] = None, stage_id: Optional[UUID4] = None, config: Optional[ProtectConfig] = None
) -> None:
    """
    Pause a stage.

    If the stage is already paused, the rulesets in the stage will not be evaluated.

    Parameters
    ----------
    project_id : Optional[UUID4], optional
        Project ID, by default None and will be taken from the config.
    stage_id : Optional[UUID4], optional
        Stage ID, by default None and will be taken from the config.
    config : Optional[ProtectConfig], optional
        Protect config, by default None and will be taken from the env vars or the local
        config file.

    Returns
    -------
    None

    Raises
    ------
    ValueError
        If the project ID is not provided or found.
    ValueError
        If the stage ID is not provided or found.
    """
    config = config or ProtectConfig.get()
    project_id = project_id or config.project_id
    stage_id = stage_id or config.stage_id
    if project_id is None:
        raise ValueError("Project ID must be provided to pause a stage.")
    if stage_id is None:
        raise ValueError("Stage ID must be provided to pause a stage.")
    config.api_client.request(
        put,
        Routes.stage.format(project_id=project_id, stage_id=stage_id),
        params=dict(action_enabled=True),
    )
    config.project_id = project_id
    config.stage_id = stage_id
    config.write()


def resume_stage(
    project_id: Optional[UUID4] = None, stage_id: Optional[UUID4] = None, config: Optional[ProtectConfig] = None
) -> None:
    config = config or ProtectConfig.get()
    project_id = project_id or config.project_id
    stage_id = stage_id or config.stage_id
    if project_id is None:
        raise ValueError("Project ID must be provided to resume a stage.")
    if stage_id is None:
        raise ValueError("Stage ID must be provided to resume a stage.")
    config.api_client.request(
        put,
        Routes.stage.format(project_id=project_id, stage_id=stage_id),
        params=dict(action_enabled=False),
    )
    config.project_id = project_id
    config.stage_id = stage_id
    config.write()
